/*
 * Copyright (c) 2017 Fraunhofer FOKUS.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the
 * GNU AGPL v3 which accompanies
 * this distribution, and is available at
 * http://www.gnu.org/licenses/agpl.txt
 *
 * Contributors:
 * Simon Dutkowski Fraunhofer FOKUS
 *
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 */
package eu.DECIDEh2020.appManager;

import eu.DECIDEh2020.appManager.models.AppDescription;
import eu.DECIDEh2020.appManager.models.Microservice;
import eu.DECIDEh2020.appManager.models.Nfr;
import eu.DECIDEh2020.appManager.models.Pattern;

import java.util.HashSet;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

public class AppDescriptionHelper {

	private AppDescription appDescription;

	public static AppDescriptionHelper create(AppDescription appDescription) {
		return new AppDescriptionHelper(appDescription);
	}

	private AppDescriptionHelper(AppDescription appDescription) {
		this.appDescription = appDescription;
	}

	public Set<String> getTags() {

		Set<String> tags;
		if (appDescription.getMicroservices() != null) {
			tags = appDescription.getMicroservices().stream().flatMap(m -> m.getTags().stream())
					.collect(Collectors.toSet());
		} else {
			tags = new HashSet<String>();
		}
		if (appDescription.getNfrs() != null) {
			Set<String> ntags = appDescription.getNfrs().stream().flatMap(m -> m.getTags().stream())
					.collect(Collectors.toSet());
			tags.addAll(ntags);
		}
		return tags;
	}

	public List<String> getMicroserviceGroups() {
		Map<String, Long> tags = appDescription.getMicroservices().stream().flatMap(m -> m.getTags().stream())
				.collect(Collectors.groupingBy(Function.identity(), Collectors.counting()));
		return tags.keySet().stream().filter(t -> tags.get(t) >= 2).collect(Collectors.toList());
	}

	public boolean isMicroserviceGroup(String tag) {
		return appDescription.getMicroservices().stream().filter(m -> m.getTags().contains(tag)).count() >= 2;
	}

	public List<Microservice> getGroupMicroservices(String tag) {
		return appDescription.getMicroservices().stream().filter(m -> m.getTags().contains(tag)).collect(Collectors.toList());
	}

	public List<Nfr> getGroupNfrs(String tag) {
		return appDescription.getNfrs().stream().filter(n -> n.getTags().contains(tag)).collect(Collectors.toList());
	}

	public List<Pattern> getGroupPatterns(String tag) {
		return appDescription.getRecommendedPatterns().stream().filter(p -> p.getTags().contains(tag)).collect(Collectors.toList());
	}

	public String getJson() throws JsonProcessingException {
		return new ObjectMapper().writerWithDefaultPrettyPrinter().writeValueAsString(appDescription);

	}

	public String getMicroserviceName(String id) {
		Microservice microservice = appDescription.getMicroservices().stream().filter(m -> m.getId().equalsIgnoreCase(id)).findFirst().orElse(null);
		return microservice != null ? microservice.getName() : null;
	}

}
