/*
 * Copyright (c) 2017 Fraunhofer FOKUS.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the
 * GNU AGPL v3 which accompanies
 * this distribution, and is available at
 * http://www.gnu.org/licenses/agpl.txt
 *
 * Contributors:
 * Simon Dutkowski Fraunhofer FOKUS
 *
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 */
package eu.DECIDEh2020.appManager;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import eu.DECIDEh2020.appManager.exceptions.DECIDEValidationException;
import eu.DECIDEh2020.appManager.models.HistoryEntry;
import org.everit.json.schema.Schema;
import org.everit.json.schema.ValidationException;
import org.everit.json.schema.loader.SchemaLoader;
import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONTokener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Path;
import java.util.Arrays;
import java.util.List;

public class HistoryFactory {

    private static Logger log = LoggerFactory.getLogger(AppDescriptionFactory.class);

    private static Schema schema;

    static {
        try (InputStream inputStream = Thread.currentThread().getContextClassLoader().getResourceAsStream("optimus_history.schema.json")) {
            JSONObject rawSchema = new JSONObject(new JSONTokener(inputStream));
            schema = SchemaLoader.load(rawSchema);
        } catch (IOException e) {
            log.error("reading schema", e);
        }

        /*
        JsonSchemaFactory factory = JsonSchemaFactory.byDefault();
        InputStream is = Thread.currentThread().getContextClassLoader().getResourceAsStream("optimus_history.schema.json");
        try {
            schema = factory.getJsonSchema(new ObjectMapper().readTree(is));
        } catch (ProcessingException | IOException e) {
            log.error("reading json schema", e);
        }
        */
    }

    public static List<HistoryEntry> loadHistory(Path path) {
        ObjectMapper mapper = new ObjectMapper();
        try {
            return Arrays.asList(mapper.readValue(path.toFile(), HistoryEntry[].class));
        } catch (IOException e) {
            log.error("reading history file", e);
            return null;
        }
   }

    public static void validateHistory(List<HistoryEntry> history) throws DECIDEValidationException {
        try {
            // must be a JSONArrat because the schema in optimus_history.schema.json declares it as an array -> otherwise the validation throws an error that a JSONObject must begin with '{'
            schema.validate(new JSONArray(new ObjectMapper().writeValueAsString(history)));
        } catch (JsonProcessingException e) {
            log.error("json to string", e);
            throw new DECIDEValidationException(e);
        } catch (ValidationException e) {
            log.error("validating json", e);
            throw new DECIDEValidationException(e);
        }
    }

    public static void validateHistory(JsonNode history) throws DECIDEValidationException {
        try {
            schema.validate(new JSONArray(history.toString()));
        } catch (ValidationException e) {
            log.error("validating json", e);
            throw new DECIDEValidationException(e);
        }
    }

}
