/*
 * Copyright (c) 2017 Fraunhofer FOKUS.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the
 * GNU AGPL v3 which accompanies
 * this distribution, and is available at
 * http://www.gnu.org/licenses/agpl.txt
 *
 * Contributors:
 * Simon Dutkowski Fraunhofer FOKUS
 *
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 */
package eu.DECIDEh2020.appManager;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import eu.DECIDEh2020.appManager.exceptions.DECIDEValidationException;
import eu.DECIDEh2020.appManager.models.AppDescription;
import org.everit.json.schema.ValidationException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.testng.annotations.Test;

import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.Path;

public class AppDescriptionFactoryTest {

    private Logger log = LoggerFactory.getLogger(getClass());

    @Test
    public void createAppDescription() {
        AppDescription appDescription = AppDescriptionFactory.createAppDescription("TestApp");
//        appDescription.getMicroservices().get(0).setPublicIP(true);
        try {
            log.debug(new ObjectMapper().writerWithDefaultPrettyPrinter().writeValueAsString(appDescription));
        } catch (JsonProcessingException e) {
            log.error("pretty print app description", e);
        }
        try {
            AppDescriptionFactory.validateAppDescription(appDescription);
            assert true;
        } catch (DECIDEValidationException e) {
            log.error("validating test resource", ((ValidationException)e.getCause()).toJSON().toString(4));
            assert false;
        }
    }

    @Test
    public void validate() {
        Path path = FileSystems.getDefault().getPath("src/test/resources/DECIDE.json");
        try {
            AppDescription appDescription = new ObjectMapper().readValue(path.toFile(), AppDescription.class);
            AppDescriptionFactory.validateAppDescription(appDescription);
            assert true;
        } catch (DECIDEValidationException e) {
            log.error("validating test resource", ((ValidationException)e.getCause()).toJSON().toString(4));
            assert false;
        } catch (IOException e) {
            log.error("reading test resource", e);
            assert false;
        }
    }

}
