/*
 * Copyright (c) 2017 Fraunhofer FOKUS.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the
 * GNU AGPL v3 which accompanies
 * this distribution, and is available at
 * http://www.gnu.org/licenses/agpl.txt
 *
 * Contributors:
 * Simon Dutkowski Fraunhofer FOKUS
 *
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 */
package eu.DECIDEh2020.appManager;

import eu.DECIDEh2020.appManager.exceptions.AppManagerException;
import eu.DECIDEh2020.appManager.exceptions.DECIDEValidationException;
import eu.DECIDEh2020.appManager.models.AppDescription;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.testng.annotations.Test;

import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Path;

public class AppManagerTest {

    private Logger log = LoggerFactory.getLogger(getClass());

    private static final String REMOTE_REPO_REF = "https://gitlab.fokus.fraunhofer.de/DECIDE/Application_Descriptor.git";
    private static final String REMOTE_REPO_USERNAME = "";
    private static final String REMOTE_REPO_PASSWORD = "";

    private static final String LOCAL_REPO_PATH = "../Application_Descriptor";

    @Test
    public void openLocalRepository() {
        Path path = FileSystems.getDefault().getPath(LOCAL_REPO_PATH);
        try {
            AppManager manager = AppManager.open(path);
            AppDescription app = manager.getAppDescription();
            app.getAdditionalProperties().forEach((key, value) -> log.debug("{} - {}", key, value));
        } catch (DECIDEValidationException e) {
            log.error("validation", e);
        } catch (AppManagerException e) {
            log.error("open repository", e);
        } catch (IOException e) {
            log.error("getting app description", e);
            assert false;
        }
    }

    @Test
    public void openRemoteRepository() {
        try {
            Path path = Files.createTempDirectory("decide_");
            AppManager manager = AppManager.open(REMOTE_REPO_REF, REMOTE_REPO_USERNAME, REMOTE_REPO_PASSWORD, path);
            AppDescription app = manager.getAppDescription();
            app.getAdditionalProperties().forEach((key, value) -> log.debug("{} - {}", key, value));
        } catch (DECIDEValidationException e) {
            log.error("validation", e);
        } catch (AppManagerException e) {
            log.error("open repository", e);
        } catch (IOException e) {
            log.error("getting app description", e);
            assert false;
        }
    }

    @Test
    public void subfolderDescriptor() {
        Path path = FileSystems.getDefault().getPath(LOCAL_REPO_PATH);
        try {
            AppManager manager = AppManager.open(path);
            manager.setDescriptor("subfolder/DECIDE.json");
            AppDescription app = manager.getAppDescription();
            app.getAdditionalProperties().forEach((key, value) -> log.debug("{} - {}", key, value));
        } catch (DECIDEValidationException e) {
            log.error("validation", e);
        } catch (AppManagerException e) {
            log.error("open repository", e);
        } catch (IOException e) {
            log.error("getting app description", e);
            assert false;
        }
    }

}
